﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Gifitti.Models;

namespace Gifitti.View_Models
{
    public partial class Form1 : Form
    {
        #region Fields
        GifModel gm;

        // GifImage _currentGif; <- used to encapsulate info later 
        private const int widthBuffer = 20;
        private const int heightBuffer = 60;
        Image globalGif;
        #endregion
        #region Constructors
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {

        }
        #endregion
        #region Buttons
        private void button1_Click(object sender, EventArgs e)
        {
            openGifFileDialog.Filter = "GIF Files|*.gif";
            DialogResult result = openGifFileDialog.ShowDialog(); // Show the dialog.
            if (result == DialogResult.OK) // Test result.
            {
                string file = openGifFileDialog.FileName;
                try
                {
                    gm = new GifModel(file);
                    gm.ReverseAtEnd = false;
                    Image loadedGif = gm.gifImage;
                    globalGif = loadedGif;
                    //gifView.Image = loadedGif;

                    gifView.Width = loadedGif.Width;
                    gifView.Height = loadedGif.Height;
                    ClientSize = new Size(loadedGif.Width + widthBuffer, loadedGif.Height + heightBuffer);
                }

                catch (IOException)
                {
                }

                CenterToScreen();

                //Focus the gif:
                gifView.Select();
                gifView.Focus();
                ActiveControl = gifView;

                //Start Animating   
                timer1.Enabled = true;

                //Enable frame selection
                textBox1.Enabled = true;
                textBox2.Enabled = true;
                button3.Enabled = true;

                //set the initial frames in textbox
                textBox1.Text = "0";
                textBox2.Text = gm.numberOfFrames.ToString();
            }
        }

        #endregion
        #region SystemChecks
        private Boolean chkImage()
        {
            Boolean check = true;
            try
            {
                int Length = globalGif.GetFrameCount(FrameDimension.Time);
            }
            catch (Exception e)
            {
                check = false;
                MessageBox.Show("Must load a gif before getting output");
            }

            return (check);
        }
        #endregion
        #region MenuItems

        private void MenuItemExportJPEG(object sender, EventArgs e)
        {
            if(chkImage())
            {
                String x = "";
                if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
                {
                    x = folderBrowserDialog.SelectedPath;
                }
                
                List<Image> IMGs = new List<Image>();
                Image test;
                int Length = globalGif.GetFrameCount(FrameDimension.Time);
                if (x != "")
                {
                    for (int i = gm.startFrame; i < gm.endFrame; i++)
                    {
                        globalGif.SelectActiveFrame(FrameDimension.Time, i);
                        test = globalGif;
                        String y = "\\tmpj-" + i + ".jpg";
                        String xy = x + y;
                        test.Save(@xy, ImageFormat.Jpeg);
                    }
                }
            }
           
        }

        private void MenuItemExportPNG(object sender, EventArgs e)
        {
            if (chkImage())
            {
                String x = "";
                if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
                {
                    x = folderBrowserDialog.SelectedPath;
                }

                List<Image> IMGs = new List<Image>();
                Image test;
                int Length = globalGif.GetFrameCount(FrameDimension.Time);
                if (x != "")
                {
                    for (int i = gm.startFrame; i < gm.endFrame; i++)
                    {
                        globalGif.SelectActiveFrame(FrameDimension.Time, i);
                        test = globalGif;
                        String y = "\\tmpp-" + i + ".png";
                        String xy = x + y;
                        test.Save(@xy, ImageFormat.Png);
                    }
                }
            }
        }

        private void MenuItemExportBMP(object sender, EventArgs e)
        {
            if (chkImage())
            {
                String x = "";
                if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
                {
                    x = folderBrowserDialog.SelectedPath;
                }

                List<Image> IMGs = new List<Image>();
                Image test;
                int Length = globalGif.GetFrameCount(FrameDimension.Time);
                if (x != "")
                {
                    for (int i = gm.startFrame; i < gm.endFrame; i++)
                    {
                        globalGif.SelectActiveFrame(FrameDimension.Time, i);
                        test = new Bitmap(globalGif);
                        String y = "\\tmpb-" + i + ".bmp";
                        String xy = x + y;
                        test.Save(@xy);
                    }
                }
            }
        }

        private void MenuItemExportTIFF(object sender, EventArgs e)
        {
            if (chkImage())
            {
                String x = "";
                if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
                {
                    x = folderBrowserDialog.SelectedPath;
                }

                List<Image> IMGs = new List<Image>();
                Image test;
                int Length = globalGif.GetFrameCount(FrameDimension.Time);
                if (x != "")
                {
                    for (int i = gm.startFrame; i < gm.endFrame; i++)
                    {
                        globalGif.SelectActiveFrame(FrameDimension.Time, i);
                        test = globalGif;
                        String y = "\\tmpt-" + i + ".tiff";
                        String xy = x + y;
                        test.Save(@xy, ImageFormat.Tiff);
                    }
                }
            }
        }
        private void button3_Click(object sender, EventArgs e)
        {
            if (timer1.Enabled == true)
            {
                button3.Text = "Start";
                gifView.Enabled = false;
                timer1.Enabled = false;
            }
            else
            {
                button3.Text = "Stop";
                gifView.Enabled = true;
                timer1.Enabled = true;
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            //get next frame
            gifView.Image = gm.GetNextFrame();
        }

        private void textBox1_TextChanged(object sender, EventArgs e)
        {
            int start = 0;
            bool pass = int.TryParse(textBox1.Text, out start);
            if (pass)
            {
                gm.startFrame = start;
            }
        }

        private void textBox2_TextChanged(object sender, EventArgs e)
        {
            int end = 0;
            bool pass = int.TryParse(textBox2.Text, out end);
            if (pass)
            {
                gm.endFrame = end;
            }

        }

        private void MenuItemFileSaveAs(object sender, EventArgs e)
        {
            if (!chkImage())
                return;
            String path = null;
            saveGifFileDialog.Filter = "GIF Files|*.gif";
            if (saveGifFileDialog.ShowDialog() == DialogResult.OK)
                path = saveGifFileDialog.FileName;
            
            //Handel Close
            if (path == null)
                return;
            //TODO temp write
            Console.WriteLine(path);
            try {
                gm.saveGif(path);
            } catch (DirectoryNotFoundException)
            {
                MessageBox.Show("Save Path Invalid!");
            }
        }

        #endregion
        #region File Dialogs
        private void openGifDialog_FileOk(object sender, CancelEventArgs e)
        {

        }
        #endregion

        private void MenuOpenFile(object sender, EventArgs e)
        {
            openGifFileDialog.Filter = "GIF Files|*.gif";
            DialogResult result = openGifFileDialog.ShowDialog(); // Show the dialog.
            if (result == DialogResult.OK) // Test result.
            {
                string file = openGifFileDialog.FileName;
                try
                {
                    gm = new GifModel(file);
                    gm.ReverseAtEnd = false;
                    Image loadedGif = gm.gifImage;
                    globalGif = loadedGif;
                    //gifView.Image = loadedGif;

                    gifView.Width = loadedGif.Width;
                    gifView.Height = loadedGif.Height;
                    ClientSize = new Size(loadedGif.Width + widthBuffer, loadedGif.Height + heightBuffer);
                }

                catch (IOException)
                {
                }

                CenterToScreen();

                //Focus the gif:
                gifView.Select();
                gifView.Focus();
                ActiveControl = gifView;

                //Start Animating   
                timer1.Enabled = true;

                //Enable frame selection
                textBox1.Enabled = true;
                textBox2.Enabled = true;
                button3.Enabled = true;

                //set the initial frames in textbox
                textBox1.Text = "0";
                textBox2.Text = gm.numberOfFrames.ToString();
            }
        }

        private void menuOpenHelp(object sender, EventArgs e)
        {
            Form2 f2 = new Form2();
            f2.Show();
        }
    }
}
